//==============================================================================
// Persistence of Vision Ray Tracer Scene File
//------------------------------------------------------------------------------
// File         : SweepSplineHair2.pov
// POV Version  : 3.7
// Description  : Utilizes Mike Williams' "SweepSpline.inc" to build parametric 
//                mesh2 spline sweeps to make hair that conforms to any given 
//                set of spline curves.
// Date         : 02/09/2014
// Author       : Robert W. McGregor, Rob@McGregorFineArt.com 
// Command-Line : +w1280 +h720 +fn16 +a0.1 +am3 +r4
//------------------------------------------------------------------------------ 

#version 3.7;

#include "colors.inc"
#include "rand.inc"
#include "transforms.inc"
#include "shapes.inc"
#include "math.inc"

default { finish { specular 0.1 roughness 0.1 diffuse 0.8 } pigment {rgb 0.5 } }

#declare Max_Hairs = 5000;

global_settings {
   max_trace_level 5
   assumed_gamma 1
   radiosity { 
      brightness      1 
      recursion_limit 3
      pretrace_start  0.08
      pretrace_end    0.01
      normal off 
      always_sample off 
   }
}
#declare R = seed(1963);

//-----------------------------------------------------------------------------
// Environment - visible background, IBL and Lights
//-----------------------------------------------------------------------------

// visible background
sky_sphere {
   pigment { 
      gradient y
      color_map { [0 rgb 0][1 rgb 0.33] } 
      rotate x*10
   }
}

// IBL
sphere { 0, 1000 hollow inverse  
   pigment { 
      image_map { exr "probe_combo_blurSmall" map_type 1 interpolate 3 } 
   }
   finish { emission 0.8 diffuse 0 }
   rotate y*240
   no_image
}

// point light
light_source { <100, 200, -100> color rgb 1.5 }

//-----------------------------------------------------------------------------
// Camera
//-----------------------------------------------------------------------------
camera { 
   location  <0, 0, -42> 
   look_at <-1.5, 5.5, 0> 
   angle 45   
   right x*image_width/image_height
}

//-----------------------------------------------------------------------------
// Something for the hair to grow on 
//-----------------------------------------------------------------------------
#include "HairBaseSurface_POV_geom.inc"
#declare HairBase = object {
   Center_Object (
      object { HairBaseSurface_ 
         scale 20 
      },x+y+z
   )                 
}
object { HairBase }

//-----------------------------------------------------------------------------
// Build some hair splines 
//-----------------------------------------------------------------------------

#include "SweepSpline.inc"

#declare oMin = min_extent(HairBase);
#declare oMax = max_extent(HairBase);

#declare Rad_Base_Min = 0.01;
#declare Rad_Base_Max = 0.02;
#declare Rad_Tip_Min  = 0.002;
#declare Rad_Tip_Max  = 0.004;

#declare Upoints = 50; // Number of points along
#declare Vpoints = 5;  // Number of points around

// profile curve (circular)
#declare VSpline = spline {
    natural_spline
   -0.25,  < 0,-1, 0>,
    0,     < 1, 0, 0>,
    0.25,  < 0, 1, 0>,
    0.5,   <-1, 0, 0>,
    0.75,  < 0,-1, 0>,
    1,     < 1, 0, 0>,
    1.25,  < 0, 1, 0>
}

#declare cnt = 0; 

#declare Hairs = union {
   //no_shadow
   #while (cnt < Max_Hairs)
      #local Tip = <RRand(oMin.x, oMax.x, R), 100, RRand(oMin.z, oMax.z, R)>;
       
      // trace it
      #declare Norm = <0, 0, 0>; 
      #declare Root  = trace(HairBase, Tip, <Tip.x, -100, Tip.z>, Norm); 
      #if (vlength(Norm)!=0) 
         #local cnt = cnt + 1;
         #local Tip = Root+Norm*RRand(9,10,R);            
         
         // array of initial linear vectors (cones)
         #local aVec = array[5];
         #local aVec[0] = Root;
         #local aVec[1] = Interpolate(2/5,0,1,Root,Tip,1);
         #local aVec[2] = Interpolate(3/5,0,1,Root,Tip,1);
         #local aVec[3] = Interpolate(4/5,0,1,Root,Tip,1);
         #local aVec[4] = Tip;
          
         // droop the straight lines into curves to make them more interesting 
         #local aVec[1] = vtransform(aVec[1], transform { rotate <RRand(5,6.25,R), RRand(5,6.25,R), RRand(5,6.25,R)> });
         #local aVec[2] = vtransform(aVec[2], transform { rotate <RRand(10,12.5,R), RRand(10,12.5,R), RRand(10,12.5,R)> });
         #local aVec[3] = vtransform(aVec[3], transform { rotate <RRand(20,25,R), RRand(20,25,R), RRand(20,25,R)> });
         #local aVec[4] = vtransform(aVec[4], transform { rotate <RRand(40,50,R), RRand(40,50,R), RRand(40,50,R)> });
        
        
         // This spline describes the path along the object
         // from control points 0 to 1: Sweep Path Curve
         
         #declare USpline = spline {
            cubic_spline
            -0.25, aVec[0],   // control point
             0.00, aVec[0],   // root
                
             0.25, aVec[1],
             0.50, aVec[2],   // center
             0.75, aVec[3],  
                
             1.00, aVec[4],   // tip
             1.25, aVec[4],   // control point
         }
         
         // This spline describes how the width varies 
         #local Rad_Base = RRand(Rad_Base_Min, Rad_Base_Max, R);
         #local Rad_Tip  = RRand(Rad_Tip_Min, Rad_Tip_Max, R);
         
         #local WSpline = spline {
           cubic_spline
            -1.0, Rad_Base,  // Control Pont
             0.0, Rad_Base,
             1.0, Rad_Tip,
             2.0, Rad_Tip    // Control Point
         }
         
         // make a mesh2 hair
         #local Filename = concat("hairs/hair_",str(cnt,0,0),".inc")
         object { 
            SweepSpline(USpline, VSpline, WSpline, Upoints, Vpoints, Filename)
            texture { uv_mapping
               pigment { 
                  gradient u
                  color_map { 
                     [0.0 srgb <RRand(0.4, 0.6, R), RRand(0, 0.2, R), RRand(0.4, 0.6, R)>]
                     [1.0 srgb <RRand(0.8, 1, R), RRand(0.8, 1, R), RRand(0, 0.2, R)>]
                  }
               }
               normal { wrinkles 0.025 scale v*100 scale RRand(0.005, 0.0075, R) }
               finish { 
                  specular 0.98 roughness 0.01 brilliance 1 metallic 0.5 
                  //reflection {0, 0.2 fresnel on} conserve_energy
               }  
            }
            interior { ior 1.5 }
         }
      #end 
   #end
}

object { Hairs }
             

